#include "AvePlane.h"
#include "ImplicitOctTree.h"

AvePlane::AvePlane(PointSet* ps, float R, ImplicitOctCell *cell){
	int* index_list;
			
	float c[3];
	cell->cellCenter(c);

	float minB[] = {c[0] - R, c[1] - R, c[2] - R};
	float maxB[] = {c[0] + R, c[1] + R, c[2] + R};

	int listN = ps->_tree->collectPointsInBox(index_list, minB, maxB);

	if(listN == 0)
		return;
	
	float (*point)[3] = ps->_point;
	float (*normal)[3] = ps->_normal;
	float *conf = ps->_conf;

	double p_ave[] = {0,0,0};
	double n_ave[] = {0,0,0};

	double totalW = 0;
	double totalW2 = 0;
	for(int i=0; i<listN; i++){
		int j = index_list[i];
		float* p = point[j];
		float* n = normal[j];

		float vx = p[0] - c[0];
		float vy = p[1] - c[1];
		float vz = p[2] - c[2];

		double w = cell->weight(sqrt(vx*vx+vy*vy+vz*vz), R);
		if(w == 0)
			continue;
		totalW2 += w;
		w *= conf[j];
		totalW += w;

		p_ave[0] += w*p[0];
		p_ave[1] += w*p[1];
		p_ave[2] += w*p[2];

		n_ave[0] += w*n[0];
		n_ave[1] += w*n[1];
		n_ave[2] += w*n[2];
	}
	if(totalW == 0)
		return;

	_conf = totalW/totalW2;

	p_ave[0] /= totalW;
	p_ave[1] /= totalW;
	p_ave[2] /= totalW;

	double len = sqrt(n_ave[0]*n_ave[0] + n_ave[1]*n_ave[1] + n_ave[2]*n_ave[2]);
	if(len != 0){
		_cx = (float)(n_ave[0]/len);
		_cy = (float)(n_ave[1]/len);
		_cz = (float)(n_ave[2]/len);
		_c0 = -(float)(_cx*p_ave[0] + _cy*p_ave[1] + _cz*p_ave[2]);
	}
	else{
		cell->_pointN = 0;
		return;
	}
		
	//max error
	/*
	_error = 0;
	for(i=0; i<listN; i++){
		int j = index_list[i];
		float* p = point[j];

		float f = fabs(value(p[0], p[1], p[2]));
		if(f > _error)
			_error = f;
	}*/

	//L2 error 
	totalW = 0;
	_error = 0;
	for(i=0; i<listN; i++){
		int j = index_list[i];
		float* p = point[j];

		float vx = p[0] - c[0];
		float vy = p[1] - c[1];
		float vz = p[2] - c[2];

		double w = conf[j]*cell->weight(sqrt(vx*vx+vy*vy+vz*vz), R);
		if(w == 0)
			continue;

		totalW += w;

		float f = value(p[0], p[1], p[2]);
		float g[3];
		gradient(g, p[0], p[1], p[2]);
		float l = g[0]*g[0] + g[1]*g[1] + g[2]*g[2];
		if(l != 0)
			_error += w*f*f/l;
		else
			_error += w*f*f;
	}
	if(totalW != 0)
		_error = sqrt(_error/totalW);
}