#include "ImplicitFunction.h"
#include <math.h>

#define UNION 0
#define INTER 1
#define DIFF 2

class BooleanImplicit: public ImplicitFunction{
public:
	ImplicitFunction* f1;
	ImplicitFunction* f2;
	bool isUnion;
	bool isReversef2;

	//From [Pasko and Savchenko 1994]
	bool blending;
	float a0, a1, a2, R;

	void setOP(int id){
		switch(id){
		case UNION:
			isUnion = true;
			isReversef2 = false;
			break;
		case INTER:
			isUnion = false;
			isReversef2 = false;
			break;
		case DIFF:
			isUnion = false;
			isReversef2 = true;
			break;
		}
	}

	float value(float x, float y, float z){
		float v1 = f1->value(x,y,z);
		float v2 = f2->value(x,y,z);

		if(isReversef2)
			v2 = -v2;

		if(!blending){
			if(isUnion){
				if(v1 > v2)
					return v1;
				else
					return v2;
				//return v1 + v2 + (float)sqrt(v1*v1 + v2*v2);
			}
			else{
				if(v1 < v2)
					return v1;
				else
					return v2;
				//return v1 + v2 - (float)sqrt(v1*v1 + v2*v2);
			}
		}
		else{
			
			float disp = a0/(1+(v1/a1)*(v1/a1)+(v2/a2)*(v2/a2));
			if(isUnion)
				return v1 + v2 + (float)sqrt(v1*v1 + v2*v2) + disp;
			else
				return v1 + v2 - (float)sqrt(v1*v1 + v2*v2) + disp;
			
			/*
			float d = (v1*v1)/(a1*a1)+(v2*v2)*(a2*a2);
			if(d > R){
				if(isUnion){
					if(v1 > v2)
						return v1;
					else
						return v2;
			
				}
				else{
					if(v1 < v2)
						return v1;
					else
						return v2;
				}
			}
			else{
				float disp;
				d = 1.5f*(d/R);
				if(d < 0.5f)
					disp = a0*(-d*d + 0.75f);
				else
					disp =  a0*(0.5f*(1.5f-d)*(1.5f-d));

				if(isUnion)
					return v1 + v2 + (float)sqrt(v1*v1 + v2*v2) + disp;
				else
					return v1 + v2 - (float)sqrt(v1*v1 + v2*v2) + disp;
			}*/
		}
	}

	void setBlenging(float a0, float a1, float a2, float R){
		this->a0 = a0;
		this->a1 = a1;
		this->a2 = a2;
		this->R = R;
		blending = true;
	}

};