#include "LocalRBF.h"
#include "jacobi.h"
#include "ImplicitOctTree.h"

LocalRBF::LocalRBF(PointSet* ps, float R, ImplicitOctCell *cell){
	int* index_list;
			
	float c[3];
	cell->cellCenter(c);

	float minB[] = {c[0] - R, c[1] - R, c[2] - R};
	float maxB[] = {c[0] + R, c[1] + R, c[2] + R};

	int listN = ps->_tree->collectPointsInBox(index_list, minB, maxB);

	cell->_pointN = listN;

	if(listN < 1){
		cell->_pointN = 0;
		return;
	}

	generateCenters(ps, index_list, listN);
	interpolateCenters();

	_error = 10;

	_support = R;
}

void LocalRBF::generateCenters(PointSet* ps, int* index_list, int listN){
	float (*point)[3] = ps->_point;
	float (*normal)[3] = ps->_normal;

	if(listN == 1){
		_centerN = 1;
		_center = new float[1][3];
		_c0 = new float[1];
		_cx = new float[1];
		_cy = new float[1];
		_cz = new float[1];

		int i = index_list[0];
		_center[0][0] = point[i][0];
		_center[0][1] = point[i][1];
		_center[0][2] = point[i][2];

		_c0[0] = -(_center[0][0]*normal[i][0] + _center[0][1]*normal[i][1] + _center[0][2]*normal[i][2]);
		_cx[0] = normal[i][0];
		_cy[0] = normal[i][1];
		_cz[0] = normal[i][2];

		return;
	}
	int split1 = split(ps, index_list, 0, listN);

	int split11 = split(ps, index_list, 0, split1);
	int split12 = split(ps, index_list, split1, listN);

	int split111 = split(ps, index_list, 0, split11);
	int split112 = split(ps, index_list, split11, split1);

	int split121 = split(ps, index_list, split1, split12);
	int split122 = split(ps, index_list, split12, listN);

	_centerN = 8;
	_center = new float[8][3];
	_c0 = new float[8];
	_cx = new float[8];
	_cy = new float[8];
	_cz = new float[8];

	int N = 0;
	float tmp_c[3], tmp_n[3];
	if(0 != split111){
		computeCenterAndNormal(tmp_c, tmp_n, ps, index_list, 0, split111);
		_center[N][0] = tmp_c[0];
		_center[N][1] = tmp_c[1];
		_center[N][2] = tmp_c[2];
		_c0[N] = -(tmp_c[0]*tmp_n[0] + tmp_c[1]*tmp_n[1] + tmp_c[2]*tmp_n[2]);
		_cx[N] = tmp_n[0];
		_cy[N] = tmp_n[1];
		_cz[N] = tmp_n[2];

		N++;
	}
	if(split111 != split11){
		computeCenterAndNormal(tmp_c, tmp_n, ps, index_list, split111, split11);
		_center[N][0] = tmp_c[0];
		_center[N][1] = tmp_c[1];
		_center[N][2] = tmp_c[2];
		_c0[N] = -(tmp_c[0]*tmp_n[0] + tmp_c[1]*tmp_n[1] + tmp_c[2]*tmp_n[2]);
		_cx[N] = tmp_n[0];
		_cy[N] = tmp_n[1];
		_cz[N] = tmp_n[2];

		N++;
	}
	if(split11 != split112){
		computeCenterAndNormal(tmp_c, tmp_n, ps, index_list, split11, split112);
		_center[N][0] = tmp_c[0];
		_center[N][1] = tmp_c[1];
		_center[N][2] = tmp_c[2];
		_c0[N] = -(tmp_c[0]*tmp_n[0] + tmp_c[1]*tmp_n[1] + tmp_c[2]*tmp_n[2]);
		_cx[N] = tmp_n[0];
		_cy[N] = tmp_n[1];
		_cz[N] = tmp_n[2];

		N++;
	}
	if(split112 != split1){
		computeCenterAndNormal(tmp_c, tmp_n, ps, index_list, split112, split1);
		_center[N][0] = tmp_c[0];
		_center[N][1] = tmp_c[1];
		_center[N][2] = tmp_c[2];
		_c0[N] = -(tmp_c[0]*tmp_n[0] + tmp_c[1]*tmp_n[1] + tmp_c[2]*tmp_n[2]);
		_cx[N] = tmp_n[0];
		_cy[N] = tmp_n[1];
		_cz[N] = tmp_n[2];

		N++;
	}
	if(split1 != split121){
		computeCenterAndNormal(tmp_c, tmp_n, ps, index_list, split1, split121);
		_center[N][0] = tmp_c[0];
		_center[N][1] = tmp_c[1];
		_center[N][2] = tmp_c[2];
		_c0[N] = -(tmp_c[0]*tmp_n[0] + tmp_c[1]*tmp_n[1] + tmp_c[2]*tmp_n[2]);
		_cx[N] = tmp_n[0];
		_cy[N] = tmp_n[1];
		_cz[N] = tmp_n[2];

		N++;
	}
	if(split121 != split12){
		computeCenterAndNormal(tmp_c, tmp_n, ps, index_list, split121, split12);
		_center[N][0] = tmp_c[0];
		_center[N][1] = tmp_c[1];
		_center[N][2] = tmp_c[2];
		_c0[N] = -(tmp_c[0]*tmp_n[0] + tmp_c[1]*tmp_n[1] + tmp_c[2]*tmp_n[2]);
		_cx[N] = tmp_n[0];
		_cy[N] = tmp_n[1];
		_cz[N] = tmp_n[2];

		N++;
	}
	if(split12 != split122){
		computeCenterAndNormal(tmp_c, tmp_n, ps, index_list, split12, split122);
		_center[N][0] = tmp_c[0];
		_center[N][1] = tmp_c[1];
		_center[N][2] = tmp_c[2];
		_c0[N] = -(tmp_c[0]*tmp_n[0] + tmp_c[1]*tmp_n[1] + tmp_c[2]*tmp_n[2]);
		_cx[N] = tmp_n[0];
		_cy[N] = tmp_n[1];
		_cz[N] = tmp_n[2];

		N++;
	}
	if(split122 != listN){
		computeCenterAndNormal(tmp_c, tmp_n, ps, index_list, split122, listN);
		_center[N][0] = tmp_c[0];
		_center[N][1] = tmp_c[1];
		_center[N][2] = tmp_c[2];
		_c0[N] = -(tmp_c[0]*tmp_n[0] + tmp_c[1]*tmp_n[1] + tmp_c[2]*tmp_n[2]);
		_cx[N] = tmp_n[0];
		_cy[N] = tmp_n[1];
		_cz[N] = tmp_n[2];

		N++;
	}

	_centerN = N;
}

void LocalRBF::interpolateCenters(){
}

int LocalRBF::split(PointSet* ps, int* index_list, int start, int end){
	if(start == end)
		return end;

	//Covarience matrix
	float **CV, d[4], **v;
	CV = new float*[4];
	v = new float*[4];
	for(int i=1; i<4; i++){
		CV[i] = new float[4];
		CV[i][1] = CV[i][2] = CV[i][3] = 0;
		v[i] = new float[4];
	}

	float (*point)[3] = ps->_point;
	float c[] = {0,0,0};
	for(i=start; i<end; i++){
		int j = index_list[i];
		c[0] += point[j][0];
		c[1] += point[j][1];
		c[2] += point[j][2];
	}
	c[0] /= end-start;
	c[1] /= end-start;
	c[2] /= end-start;

	for(i=start; i<end; i++){
		int j = index_list[i];
		float *p = point[j];
		float x = p[0] - c[0];
		float y = p[1] - c[1];
		float z = p[2] - c[2];

		CV[1][1] += x*x;
		CV[1][2] += x*y;
		CV[1][3] += x*z;
		CV[2][1] += y*x;
		CV[2][2] += y*y;
		CV[2][3] += y*z;
		CV[3][1] += z*x;
		CV[3][2] += z*y;
		CV[3][3] += z*z;
	}

	int nrot;
	Jacobi ja;
	ja.jacobi(CV, 3, d ,v, &nrot);
			
	int maxi;
	if(d[1] > d[2])
		maxi = 1;
	else
		maxi = 2;
	if(d[maxi] < d[3])
		maxi = 3;

	float sn[3];
	sn[0] = v[1][maxi];
	sn[1] = v[2][maxi];
	sn[2] = v[3][maxi];

	for(i=1; i<4; i++){
		delete[] CV[i];
		delete[] v[i];
	}
	delete[] CV;
	delete[] v;

	i = start;
	int j = end-1;
	while(i <= j){
		float* p = point[index_list[i]];
		float dot = (p[0]-c[0])*sn[0] + (p[1]-c[1])*sn[1] + (p[2]-c[2])*sn[2];

		if(dot < 0)
			i++;
		else{
			int tmp = index_list[i];
			index_list[i] = index_list[j];
			index_list[j] = tmp;
			j--;
		}
	}
	return i;
}

void LocalRBF::computeCenterAndNormal(float c[], float n[], PointSet* ps, 
									  int* index_list, int start, int end){
	float (*point)[3] = ps->_point;
	float (*normal)[3] = ps->_normal;

	c[0] = c[1] = c[2] =
	n[0] = n[1] = n[2] = 0;
	for(int i=start; i<end; i++){
		int j = index_list[i];
		float* p = point[j];
		float* m = normal[j];

		c[0] += p[0];
		c[1] += p[1];
		c[2] += p[2];

		n[0] += m[0];
		n[1] += m[1];
		n[2] += m[2];
	}
	int N = end - start;
	c[0] /= N;
	c[1] /= N;
	c[2] /= N;

	float l = sqrt(n[0]*n[0] + n[1]*n[1] + n[2]*n[2]);
	if(l != 0){
		n[0] /= l;
		n[1] /= l;
		n[2] /= l;
	}
}