#ifndef LOCALRBF_H
#define LOCALRBF_H 

#include <stdio.h>
#include <math.h>
#include "PointSet.h"
//#include "ImplicitOctTree.h"
#include "LocalApprox.h"

class ImplicitOctCell;

class LocalRBF: public LocalApprox{

public:
	float _support;
	float *_c0, *_cx, *_cy, *_cz;
	int _centerN;
	float (*_center)[3];

	LocalRBF(PointSet* ps, float R, ImplicitOctCell *cell);

	~LocalRBF(){
		delete[] _c0;
		delete[] _cx;
		delete[] _cy;
		delete[] _cz;
		delete[] _center;
	}

	float weight(int index, float x, float y, float z){
		float *c = _center[index];
		float vx = x - c[0];
		float vy = y - c[1];
		float vz = z - c[2];
		float r = (float)sqrt(vx*vx + vy*vy + vz*vz);
		if(r > _support)
			return 0;
		r /= _support;
		return (1.0f-r)*(1.0f-r);
	}

	float value(float x, float y, float z){
		float f = 0;
		for(int i=0; i<_centerN; i++)
			f += (_c0[i] + _cx[i]*x + _cy[i]*y + _cz[i]*z)*weight(i, x, y, z);
		return f;
	}

	void gradient(float g[], float x, float y, float z){
		
	}

	void generateCenters(PointSet* ps, int* index_list, int listN);
	void interpolateCenters();
	int split(PointSet* ps, int* index_list, int start, int end);
	void computeCenterAndNormal(float c[], float n[], PointSet* ps, 
								int* index_list, int start, int end);
};

#endif