#include "QuadricB.h"
#include "ImplicitOctTree.h"
#include "LU.h"

//Quadric computed by Both side points
QuadricB::QuadricB(PointSet* ps, float R, ImplicitOctCell *cell){
		int* index_list;
			
		float *c = cell->_center;

		float minB[] = {c[0] - R, c[1] - R, c[2] - R};
		float maxB[] = {c[0] + R, c[1] + R, c[2] + R};

		int listN = ps->_tree->collectPointsInBox(index_list, minB, maxB);
		
		cell->_pointN = listN;

		if(listN == 0)
			return;
		
		float (*point)[3] = ps->_point;
		float (*normal)[3] = ps->_normal;

		double p_ave[] = {0,0,0};
		double n_ave[] = {0,0,0};

		//For LU
		float** A = new float*[11];
		for(int i=1; i<11; i++){
			A[i] = new float[11];
			A[i][1] = A[i][2] = A[i][3] = 
		    A[i][4] = A[i][5] = A[i][6] =
			A[i][7] = A[i][8] = A[i][9] = A[i][10] = 0;
		}
		int* index = new int[11];
		float* d = new float[11];
		double* b = new double[11];
		b[1] = b[2] = b[3] = b[4] = b[5] = b[6] = b[7] = b[8] = b[9] = b[10] = 0;

		float off = R*OFF;
		int count = 0;
		for(i=0; i<listN; i++){
			int in = index_list[i];
			float* p = point[in];
			float* n = normal[in];

			float x[10];

			//+ offset
			x[0] = 1;
			x[1] = p[0] + off*n[0];
			x[2] = p[1] + off*n[1];
			x[3] = p[2] + off*n[2];
			x[4] = x[1]*x[1];
			x[5] = x[2]*x[2];
			x[6] = x[3]*x[3];
			x[7] = x[1]*x[2];
			x[8] = x[2]*x[3];
			x[9] = x[3]*x[1];

			float vx = x[1] - c[0];
			float vy = x[2] - c[1];
			float vz = x[3] - c[2];
			double w =cell->weight(sqrt(vx*vx+vy*vy+vz*vz), R);
			if(w != 0){
				count++;
				w *= w;

				for(int j=0; j<10; j++){
					for(int k=0; k<10; k++)
						A[j+1][k+1] += w*x[j]*x[k];
					b[j+1] += w*off*x[j];
				}
			}

			//- offset
			x[0] = 1;
			x[1] = p[0] - off*n[0];
			x[2] = p[1] - off*n[1];
			x[3] = p[2] - off*n[2];
			x[4] = x[1]*x[1];
			x[5] = x[2]*x[2];
			x[6] = x[3]*x[3];
			x[7] = x[1]*x[2];
			x[8] = x[2]*x[3];
			x[9] = x[3]*x[1];

			vx = x[1] - c[0];
			vy = x[2] - c[1];
			vz = x[3] - c[2];
			w = cell->weight(sqrt(vx*vx+vy*vy+vz*vz), R);
			if(w != 0){
				count++;
				w *= w;

				for(int j=0; j<10; j++){
					for(int k=0; k<10; k++)
						A[j+1][k+1] += w*x[j]*x[k];
					b[j+1] -= w*off*x[j];
				}
			}
		}
		cell->_pointN = count;
		if(count < 20){
			for(i = i=1; i<11; i++)
				delete[] A[i];
			delete[] A;
			delete[] index;
			delete[] d;
			delete[] b;
			return;
		}

		LU lu;
		lu.ludcmp(A, 10, index, d);
		lu.lubksb(A, 10, index, b);

		_c0 = b[1];

		_cx = b[2];
		_cy = b[3];
		_cz = b[4];

		_cxx = b[5];
		_cyy = b[6];
		_czz = b[7];

		_cxy = b[8];
		_cyz = b[9];
		_czx = b[10];

		for(i = i=1; i<11; i++)
			delete[] A[i];
		delete[] A;
		delete[] index;
		delete[] d;
		delete[] b;

		//normalize function
		float g[3];
		gradient(g, c[0], c[1], c[2]);
		double len = sqrt(g[0]*g[0] + g[1]*g[1] + g[2]*g[2]);
		if(len != 0){
			_cxy = (float)(_cxy/len);
			_cyz = (float)(_cyz/len);
			_czx = (float)(_czx/len);

			_cxx = (float)(_cxx/len);
			_cyy = (float)(_cyy/len);
			_czz = (float)(_czz/len);

			_cx = (float)(_cx/len);
			_cy = (float)(_cy/len);
			_cz = (float)(_cz/len);
			_c0 = (float)(_c0/len);
		}

		_error = 0;
		for(i=0; i<listN; i++){
			int j = index_list[i];
			float* p = point[j];

			float vx = p[0] - c[0];
			float vy = p[1] - c[1];
			float vz = p[2] - c[2];

			double w = cell->weight(sqrt(vx*vx+vy*vy+vz*vz), R);

			float f = fabs(value(p[0], p[1], p[2]));
			float g[3];
			gradient(g, p[0], p[1], p[2]);
			float e = f/sqrt(g[0]*g[0] + g[1]*g[1] + g[2]*g[2]);
			if(e > _error)
				_error = e;
		}
}